@@ -1,6 +1,6 @@
 /* sig.c - interface for shell signal handlers and signal initialization. */
 
-/* Copyright (C) 1994-2015 Free Software Foundation, Inc.
+/* Copyright (C) 1994-2010 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -43,11 +43,9 @@
 #include "trap.h"
 
 #include "builtins/common.h"
-#include "builtins/builtext.h"
 
 #if defined (READLINE)
 #  include "bashline.h"
-#  include <readline/readline.h>
 #endif
 
 #if defined (HISTORY)
@@ -57,7 +55,6 @@
 extern int last_command_exit_value;
 extern int last_command_exit_signal;
 extern int return_catch_flag;
-extern int running_trap;
 extern int loop_level, continuing, breaking, funcnest;
 extern int executing_list;
 extern int comsub_ignore_return;
@@ -65,28 +62,17 @@ extern int parse_and_execute_level, shell_initialized;
 #if defined (HISTORY)
 extern int history_lines_this_session;
 #endif
-extern int no_line_editing;
-extern int wait_signal_received;
-extern int wait_intr_flag;
-extern sh_builtin_func_t *this_shell_builtin;
 
 extern void initialize_siglist ();
 
-#if !defined (JOB_CONTROL)
-extern void initialize_job_signals __P((void));
-#endif
-
 /* Non-zero after SIGINT. */
-volatile sig_atomic_t interrupt_state = 0;
+volatile int interrupt_state = 0;
 
 /* Non-zero after SIGWINCH */
-volatile sig_atomic_t sigwinch_received = 0;
-
-/* Non-zero after SIGTERM */
-volatile sig_atomic_t sigterm_received = 0;
+volatile int sigwinch_received = 0;
 
 /* Set to the value of any terminating signal received. */
-volatile sig_atomic_t terminating_signal = 0;
+volatile int terminating_signal = 0;
 
 /* The environment at the top-level R-E loop.  We use this in
    the case of error return. */
@@ -272,7 +258,7 @@ initialize_terminating_signals ()
       if (interactive_shell == 0 && XHANDLER (i) == SIG_IGN)
 	{
 	  sigaction (XSIG (i), &oact, &act);
-	  set_signal_hard_ignored (XSIG (i));
+	  set_signal_ignored (XSIG (i));
 	}
 #if defined (SIGPROF) && !defined (_MINIX)
       if (XSIG (i) == SIGPROF && XHANDLER (i) != SIG_DFL && XHANDLER (i) != SIG_IGN)
@@ -296,7 +282,7 @@ initialize_terminating_signals ()
       if (interactive_shell == 0 && XHANDLER (i) == SIG_IGN)
 	{
 	  signal (XSIG (i), SIG_IGN);
-	  set_signal_hard_ignored (XSIG (i));
+	  set_signal_ignored (XSIG (i));
 	}
 #ifdef SIGPROF
       if (XSIG (i) == SIGPROF && XHANDLER (i) != SIG_DFL && XHANDLER (i) != SIG_IGN)
@@ -331,9 +317,7 @@ initialize_shell_signals ()
   if (interactive)
     {
       set_signal_handler (SIGINT, sigint_sighandler);
-      get_original_signal (SIGTERM);
-      if (signal_is_hard_ignored (SIGTERM) == 0)
-	set_signal_handler (SIGTERM, sigterm_sighandler);
+      set_signal_handler (SIGTERM, SIG_IGN);
       set_sigwinch_handler ();
     }
 }
@@ -372,8 +356,6 @@ reset_terminating_signals ()
       signal (XSIG (i), XHANDLER (i));
     }
 #endif /* !HAVE_POSIX_SIGNALS */
-
-  termsigs_initialized = 0;
 }
 #undef XSIG
 #undef XHANDLER
@@ -394,7 +376,7 @@ top_level_cleanup ()
 
   run_unwind_protects ();
   loop_level = continuing = breaking = funcnest = 0;
-  executing_list = comsub_ignore_return = return_catch_flag = wait_intr_flag = 0;
+  executing_list = comsub_ignore_return = return_catch_flag = 0;
 }
 
 /* What to do when we've been interrupted, and it is safe to handle it. */
@@ -405,8 +387,6 @@ throw_to_top_level ()
 
   if (interrupt_state)
     {
-      if (last_command_exit_value < 128)
-	last_command_exit_value = 128 + SIGINT;
       print_newline = 1;
       DELINTERRUPT;
     }
@@ -418,24 +398,19 @@ throw_to_top_level ()
 				(last_command_exit_value - 128) : 0;
   last_command_exit_value |= 128;
 
-  /* Run any traps set on SIGINT, mostly for interactive shells */
-  if (signal_is_trapped (SIGINT))
-    run_interrupt_trap (1);
+  /* Run any traps set on SIGINT. */
+  run_interrupt_trap ();
 
   /* Clean up string parser environment. */
   while (parse_and_execute_level)
     parse_and_execute_cleanup ();
 
-  if (running_trap > 0)
-    run_trap_cleanup (running_trap - 1);
-
 #if defined (JOB_CONTROL)
   give_terminal_to (shell_pgrp, 0);
 #endif /* JOB_CONTROL */
 
 #if defined (JOB_CONTROL) || defined (HAVE_POSIX_SIGNALS)
-  /* This needs to stay because jobs.c:make_child() uses it without resetting
-     the signal mask. */
+  /* This should not be necessary on systems using sigsetjmp/siglongjmp. */
   sigprocmask (SIG_SETMASK, &top_level_mask, (sigset_t *)NULL);
 #endif
 
@@ -452,7 +427,7 @@ throw_to_top_level ()
 
   run_unwind_protects ();
   loop_level = continuing = breaking = funcnest = 0;
-  executing_list = comsub_ignore_return = return_catch_flag = wait_intr_flag = 0;
+  executing_list = comsub_ignore_return = return_catch_flag = 0;
 
   if (interactive && print_newline)
     {
@@ -474,7 +449,7 @@ void
 jump_to_top_level (value)
      int value;
 {
-  sh_longjmp (top_level, value);
+  longjmp (top_level, value);
 }
 
 sighandler
@@ -530,25 +505,12 @@ termsig_sighandler (sig)
     {
 #if defined (HISTORY)
       /* XXX - will inhibit history file being written */
-#  if defined (READLINE)
-      if (interactive_shell == 0 || interactive == 0 || (sig != SIGHUP && sig != SIGTERM) || no_line_editing || (RL_ISSTATE (RL_STATE_READCMD) == 0))
-#  endif
-        history_lines_this_session = 0;
+      history_lines_this_session = 0;
 #endif
       terminate_immediately = 0;
       termsig_handler (sig);
     }
 
-#if defined (READLINE)
-  /* Set the event hook so readline will call it after the signal handlers
-     finish executing, so if this interrupted character input we can get
-     quick response.  If readline is active or has modified the terminal we
-     need to set this no matter what the signal is, though the check for
-     RL_STATE_TERMPREPPED is possibly redundant. */
-  if (RL_ISSTATE (RL_STATE_SIGHANDLER) || RL_ISSTATE (RL_STATE_TERMPREPPED))
-    bashline_set_event_hook ();
-#endif
-
   SIGRETURN (0);
 }
 
@@ -568,20 +530,13 @@ termsig_handler (sig)
 
   /* I don't believe this condition ever tests true. */
   if (sig == SIGINT && signal_is_trapped (SIGINT))
-    run_interrupt_trap (0);
+    run_interrupt_trap ();
 
 #if defined (HISTORY)
-  /* If we don't do something like this, the history will not be saved when
-     an interactive shell is running in a terminal window that gets closed
-     with the `close' button.  We can't test for RL_STATE_READCMD because
-     readline no longer handles SIGTERM synchronously.  */
-  if (interactive_shell && interactive && (sig == SIGHUP || sig == SIGTERM) && remember_on_history)
+  if (interactive_shell && sig != SIGABRT)
     maybe_save_shell_history ();
 #endif /* HISTORY */
 
-  if (this_shell_builtin == read_builtin)
-    read_tty_cleanup ();
-
 #if defined (JOB_CONTROL)
   if (sig == SIGHUP && (interactive || (subshell_environment & (SUBSHELL_COMSUB|SUBSHELL_PROCSUB))))
     hangup_all_jobs ();
@@ -594,9 +549,9 @@ termsig_handler (sig)
 
   /* Reset execution context */
   loop_level = continuing = breaking = funcnest = 0;
-  executing_list = comsub_ignore_return = return_catch_flag = wait_intr_flag = 0;
+  executing_list = comsub_ignore_return = return_catch_flag = 0;
 
-  run_exit_trap ();	/* XXX - run exit trap possibly in signal context? */
+  run_exit_trap ();
   set_signal_handler (sig, SIG_DFL);
   kill (getpid (), sig);
 }
@@ -615,29 +570,12 @@ sigint_sighandler (sig)
   if (interrupt_state == 0)
     ADDINTERRUPT;
 
-  /* We will get here in interactive shells with job control active; allow
-     an interactive wait to be interrupted.  wait_intr_flag is only set during
-     the execution of the wait builtin and when wait_intr_buf is valid. */
-  if (wait_intr_flag)
-    {
-      last_command_exit_value = 128 + sig;
-      wait_signal_received = sig;
-      SIGRETURN (0);
-    }
-      
   if (interrupt_immediately)
     {
       interrupt_immediately = 0;
       last_command_exit_value = 128 + sig;
       throw_to_top_level ();
     }
-#if defined (READLINE)
-  /* Set the event hook so readline will call it after the signal handlers
-     finish executing, so if this interrupted character input we can get
-     quick response. */
-  else if (RL_ISSTATE (RL_STATE_SIGHANDLER))
-    bashline_set_event_hook ();
-#endif
 
   SIGRETURN (0);
 }
@@ -671,17 +609,10 @@ unset_sigwinch_handler ()
 #endif
 }
 
-sighandler
-sigterm_sighandler (sig)
-     int sig;
-{
-  sigterm_received = 1;		/* XXX - counter? */
-  SIGRETURN (0);
-}
-
 /* Signal functions used by the rest of the code. */
 #if !defined (HAVE_POSIX_SIGNALS)
 
+#if defined (JOB_CONTROL)
 /* Perform OPERATION on NEWSET, perhaps leaving information in OLDSET. */
 sigprocmask (operation, newset, oldset)
      int operation, *newset, *oldset;
@@ -700,7 +631,7 @@ sigprocmask (operation, newset, oldset)
       break;
 
     case SIG_SETMASK:
-      old = sigsetmask (new);
+      sigsetmask (new);
       break;
 
     default:
@@ -710,6 +641,7 @@ sigprocmask (operation, newset, oldset)
   if (oldset)
     *oldset = old;
 }
+#endif /* JOB_CONTROL */
 
 #else
 
@@ -734,20 +666,12 @@ set_signal_handler (sig, handler)
   /* XXX - bash-4.2 */
   /* We don't want a child death to interrupt interruptible system calls, even
      if we take the time to reap children */
-#if defined (SIGCHLD)
   if (sig == SIGCHLD)
     act.sa_flags |= SA_RESTART;		/* XXX */
-#endif
-  /* If we're installing a SIGTERM handler for interactive shells, we want
-     it to be as close to SIG_IGN as possible. */
-  if (sig == SIGTERM && handler == sigterm_sighandler)
-    act.sa_flags |= SA_RESTART;		/* XXX */
 
   sigemptyset (&act.sa_mask);
   sigemptyset (&oact.sa_mask);
-  if (sigaction (sig, &act, &oact) == 0)
-    return (oact.sa_handler);
-  else
-    return (SIG_DFL);
+  sigaction (sig, &act, &oact);
+  return (oact.sa_handler);
 }
 #endif /* HAVE_POSIX_SIGNALS */
